void updateButtons() // based on https://blog.adafruit.com/2009/10/20/example-code-for-multi-button-checker-with-debouncing/
{
	static byte previousstate[NUMBUTTONS];
	static byte currentstate[NUMBUTTONS];
	static long lasttime;
	byte index;

	if (millis() < lasttime)
	{
		// we wrapped around, lets just try again
		lasttime = millis();
	}

	if ((lasttime + DEBOUNCE) > millis()) {
		// not enough time has passed to debounce
		return;
	}
	// ok we have waited DEBOUNCE milliseconds, lets reset the timer
	lasttime = millis();

	for (index = 0; index < NUMBUTTONS; index++) // when we start, we clear out the "just" indicators
	{
		justreleased[index] = 0;
		justpressed[index] = 0;

		currentstate[index] = digitalRead(buttons[index]);   // read the button

		if (currentstate[index] == previousstate[index]) {
			if ((pressed[index] == LOW) && (currentstate[index] == LOW)) {
				// just pressed
				justpressed[index] = 1;
			}
			else if ((pressed[index] == HIGH) && (currentstate[index] == HIGH)) {
				// just released
				justreleased[index] = 1;
			}
			pressed[index] = !currentstate[index];  // remember, digital HIGH means NOT pressed
		}

		previousstate[index] = currentstate[index];   // keep a running tally of the buttons
	}
}

void clearJust()
{
	for (byte index = 0; index < NUMBUTTONS; index++) // when we start, we clear out the "just" indicators
	{
		justreleased[index] = 0;
		justpressed[index] = 0;
	}
}

void handleButtons()
{
	uint8_t _mode = 99;

	if (justreleased[0])
	{

		if (ignoreF1Released)
		{
			ignoreF1Released = false;
			if (pitchBend)
			{
				pitchBendOff();
				resetPitch();
				sendBend(0);
			}
		}
		else if (pitchBend)
		{
			pitchBendOff();
			resetPitch();
			sendBend(0);
		}
		else
		{
			if (mode == 10) // in case of save
				_mode = 0;
			else if (mode == 11) // in case of save confirm
			{
			}
			else
				_mode = (mode > 0) ? mode - 1 : numModes - 1;
		}
	}

	if (justreleased[1])
	{
		if (ignoreF2Released)
			ignoreF2Released = false;
		else
		{
			if (mode >= numModes) // in case of save and save confirm
				_mode = 0;
			else
				_mode = (mode + 1 < numModes) ? mode + 1 : 0;
		}
	}

	if (_mode != 99)
	{

		changeMode(_mode, true);
		clearJust();
		return; // mode change - get out of handleButtons
	}

	if (justpressed[0] && mode == 11) // cancel save on save confirm page
	{
		changeMode(10, true);
		ignoreF1Released = true;
		clearJust();
		return;
	}

	if (justpressed[1])
	{
		if (mode == 11) // save on save confirm page
		{
			savePart();
			ignoreF2Released = true;
		}
		else // start the timer to go into save mode
			saveTimer = millis();
	}

	// check justpressed for normal functions
	if (justpressed[2])
	{
		param = (param - 1 >= 0) ? param - 1 : numParams - 1;
		setLastParam();
		changeMode(mode, false); // call changeMode in case the parameter page has changed
		showAlert(paramName[param % 3], 400, 0, 0, 1);
	}

	if (justpressed[3])
	{
		param = (param + 1 < numParams) ? param + 1 : 0;
		setLastParam();
		changeMode(mode, false);  // call changeMode in case the parameter page has changed
		showAlert(paramName[param % 3], 400, 0, 0, 1);
	}

	if (justpressed[4])
	{
		decreaseValue();
	}
	if (justpressed[5])
	{
		increaseValue();
	}
	clearJust();

	// FUNCTION BUTTONS
	if (pressed[0])
		F1 = true;
	else
		F1 = false;

	if (pressed[1])
	{
		F2 = true;
		if (saveTimer != 0 && millis() - saveTimer > 400) // after 400ms
		{
			if (mode == 2) // settings page
				saveSettings();
			else if (mode != 10)
				changeMode(10, true);
			else if (mode == 10)
				savePart();
			ignoreF2Released = true;
			saveTimer = 0;
		}
	}
	else
		F2 = false;
}

void setLastParam()
{
	if (mode < 3)
		lastParam[mode] = param;
	else if (mode == 10)
		lastParam[3] = param;
}

void increaseValue()
{
	*adjustValuePtr[param % 3] += adjustIncrement[param % 3];
	if (*adjustValuePtr[param % 3] > upperLimit[param % 3]) // wrap around
		*adjustValuePtr[param % 3] = lowerLimit[param % 3];

	// check if a function needs to be called
	checkButtonFunctions();
}

void decreaseValue()
{
	*adjustValuePtr[param % 3] -= adjustIncrement[param % 3];
	if (*adjustValuePtr[param % 3] < lowerLimit[param % 3]) // wrap around
		*adjustValuePtr[param % 3] = upperLimit[param % 3];

	// check if a function needs to be called
	checkButtonFunctions();
}

void checkButtonFunctions()
{
	switch (mode)
	{
	case 0: // PLAY
		if (adjustValuePtr[param] == &current_waveform)
			setWaveform(true);
		if (adjustValuePtr[param] == &currentScale)
			setScale(true);
		if (adjustValuePtr[param] == &rootnote)
			setRoot(true);
		break;

	case 1: // SONG
		if (adjustValuePtr[param] == &song)
			setSong();
		if (adjustValuePtr[param] == &part)
			setPart();
		break;
	case 2: // SETTINGS
		settingsAreSaved = false;
		switch (param / 3) // which parameter page are we on
		{
		case 0:
			if (adjustValuePtr[param % 3] == &settings.pitchTofIsRight)
				setSide();
			if (adjustValuePtr[param % 3] == &settings.pitchMaxMM)
				setPitchMaxMM();
			if (adjustValuePtr[param % 3] == &settings.volMaxMM)
				setVolMaxMM();
			break;
		case 1:
			if (adjustValuePtr[param % 3] == &settings.pitchNumSteps)
				setPitchNumSteps();
			if (adjustValuePtr[param % 3] == &settings.pitchMidiChan)
				setPitchChn();
			if (adjustValuePtr[param % 3] == &settings.pitchMidiCC)
				setPitchCC();
			break;
		case 2:
			if (adjustValuePtr[param % 3] == &settings.volMidiChan)
				setVolChn();
			if (adjustValuePtr[param % 3] == &settings.volMidiCC)
				setVolCC();
			if (adjustValuePtr[param % 3] == &settings.ledMode)
				setLedMode();
			break;
		case 3:
			if (adjustValuePtr[param % 3] == &settings.bendRange)
				setPitchBendRange();
			if (adjustValuePtr[param % 3] == &settings.isMuted)
				setMuted();
			break;
		}
		break;
	case 10: // SAVE
		if (adjustValuePtr[param] == &songToSave)
			setSongToSave();
		if (adjustValuePtr[param] == &partToSave)
			setPartToSave();
		break;
	}
}

void pitchBendOff()
{
	pitchBend = false;
	bendCenterMM = 0;
	updateOLED = true; // update the display to remove the pitchbend icon
}