void refresh()
{
	// picture loop  
	u8g2.clearBuffer();
	draw();
	u8g2.sendBuffer();
	if (splash)
	{
		delay(500);
		splash = false;
	}
}

void draw()
{
	if (splash)
		showSplash();
	else if (alert > 0)
		doAlert();
	else
		showMode();
}

void u8g2_prepare(void) {
	u8g2.setFont(u8g2_font_profont15_tf);
	u8g2.setFontRefHeightExtendedText();
	u8g2.setDrawColor(1);
	u8g2.setFontPosTop();
	u8g2.setFontDirection(0);
}

void modeBox(int box, String paramLabel, int8_t spacing) // box with text - use spacing 100+ for tight vertical spacing
{
	if (spacing < 100)
	{
		switch (box)
		{
		case 0:
			u8g2.setDrawColor(1);
			u8g2.drawRBox(0, 1, 42, 18, 3);
			u8g2.setDrawColor(0);
			u8g2.drawRBox(1, 2, 40, 16, 3);
			u8g2.setDrawColor(1);
			u8g2.setCursor(5 + spacing, 3);
			u8g2.setFont(u8g2_font_profont15_tf);
			break;
		}
		u8g2.print(paramLabel);
	}
}

void showAlert(String _alertString, uint16_t _alertDur, uint8_t _xPos, uint8_t _boxWidth, uint8_t size)
{
	alert = _alertDur;
	alertString = _alertString;
	xPos = _xPos;
	boxWidth = _boxWidth;
	alertSize = size;
	updateOLED = true;
}

void doAlert()
{
	if (boxWidth > 0)
	{
		u8g2.setDrawColor(1);
		u8g2.drawRBox(xPos, 17, boxWidth, 33, 3);
		u8g2.setDrawColor(0);
		u8g2.drawRBox(xPos + 1, 18, boxWidth - 2, 31, 3);
	}
	u8g2.setDrawColor(1);
	switch (alertSize)
	{
	case 0:
		u8g2.setFont(u8g2_font_fub14_tf);
		break;
	case 1:
		u8g2.setFont(u8g2_font_fub20_tf);
		break;
	case 2:
		u8g2.setFont(u8g2_font_fub35_tf);
		break;
	case 10:
		//u8g2.setFont(u8g2_font_emoticons21_tr);
		break;
	}
	u8g2.setCursor(xPos + 5, 22);
	u8g2.print(alertString);
}

void showMode()
{
	// MODE NAME
	//u8g2.setFont(u8g2_font_profont17_tf);
	u8g2.setFont(u8g2_font_fub14_tf);
	u8g2.setCursor(0, 3);
	u8g2.print(modeName);

	// PARAM HIGHLIGHT
	if (mode != 11) // no  highlight on confirm page
		u8g2.drawRBox(0, 21 + ((param % 3) * 13), 42, 13, 2);

	// PARAM 1
	if ((param % 3) == 0)
		u8g2.setDrawColor(0);
	else
		u8g2.setDrawColor(1);
	u8g2.setFont(u8g2_font_crox1hb_tf);
	u8g2.setCursor(0, 22);
	u8g2.print(paramName[0]);
	u8g2.setDrawColor(1);
	u8g2.setFont(u8g2_font_crox1h_tf);
	u8g2.setCursor(45, 22);
	u8g2.print(*paramVal[0]);

	// PARAM 2
	//if (!(mode == 2 && param / 3 == 3)) // keep the second line of the last settings page blank
	//{
	if ((param % 3) == 1)
		u8g2.setDrawColor(0);
	else
		u8g2.setDrawColor(1);
	u8g2.setFont(u8g2_font_crox1hb_tf);
	u8g2.setCursor(0, 35);
	u8g2.print(paramName[1]);
	u8g2.setDrawColor(1);
	u8g2.setFont(u8g2_font_crox1h_tf);
	u8g2.setCursor(45, 35);
	u8g2.print(*paramVal[1]);
	//}

	// PARAM 3
	if (mode != 1)
	{
		if ((param % 3) == 2)
			u8g2.setDrawColor(0);
		else
			u8g2.setDrawColor(1);
		u8g2.setFont(u8g2_font_crox1hb_tf);
		u8g2.setCursor(0, 48);
		u8g2.print(paramName[2]);
	}
	else
	{
		u8g2.setDrawColor(1);
		u8g2.setFont(u8g2_font_open_iconic_arrow_2x_t);
		u8g2.setCursor(25, 48);
		u8g2.print("Z"); // different right arrow styles... B, F, J, N, R, Z
	}

	if (!(mode == 2 && param / 3 == 3)) // don't show a parameter value for the 3rd parameter on the last settings page
	{
		u8g2.setDrawColor(1);
		u8g2.setFont(u8g2_font_crox1h_tf);
		u8g2.setCursor(45, 48);
		u8g2.print(*paramVal[2]);
	}

	if (mode != 2 && pitchBend && currentScale != numScales) // don't show on settings page or in free pitch/theremin mode
	{
		u8g2.setDrawColor(1);
		u8g2.setFont(u8g2_font_open_iconic_all_2x_t);
		u8g2.setCursor(110, 1);
		u8g2.print(char(238)); // different right arrow styles... B, F, J, N, R, Z, see https://github.com/olikraus/u8g2/wiki/fntgrpiconic#open_iconic_all_1x
	}

	if (mode == 2)
	{
		u8g2.setDrawColor(1); // page number
		String pageNum = "/4";
		String pageNumber = String(param / 3 + 1) + pageNum;
		u8g2.setFont(u8g2_font_synchronizer_nbp_tf);
		u8g2.setCursor(108, 18);
		u8g2.print(pageNumber);

		if (!settingsAreSaved) // draw a star as reminder that settings are not saved
		{
			u8g2.setFont(u8g2_font_open_iconic_weather_2x_t);
			u8g2.setCursor(110, 1);
			u8g2.print(char(68));
		}
	}

	if (settings.isMuted) // draw the mute icon
	{
		u8g2.setDrawColor(1);
		u8g2.setFont(u8g2_font_open_iconic_play_2x_t);
		u8g2.setCursor(86, 1);
		u8g2.print(char(81));
		u8g2.setDrawColor(1);
		u8g2.setFont(u8g2_font_open_iconic_check_1x_t);
		u8g2.setCursor(100, 5);
		u8g2.print(char(68));
	}
}

void showSplash()
{
	// splash
	u8g2.drawXBMP(0, 0, TheremonSplash_width, TheremonSplash_height, TheremonSplash_bits);
	u8g2.setDrawColor(1);
	u8g2.setFont(u8g2_font_synchronizer_nbp_tf);
	u8g2.setCursor(93, 18);
	u8g2.print(versionNumber);

	// icons
	static char icon[38] = {71, 84, 85, 90, 91, 92, 93, 94, 96, 100, 103, 104, 105, 106, 107, 108, 129, 134, 147, 152, 156, 165, 166, 168, 170, 182, 183, 184, 210, 211, 212, 213, 214, 215, 216, 217, 218, 225  };
	randomSeed(analogRead(2)); // analog pin A2 is unconnected so it should return a random value 
	u8g2.setFont(u8g2_font_open_iconic_all_1x_t);
	u8g2.setCursor(70, 32);
	u8g2.print(char(icon[random(0, 38)]));
	u8g2.setCursor(85, 32);
	u8g2.print(char(icon[random(0, 38)]));
	u8g2.setCursor(100, 32);
	u8g2.print(char(icon[random(0, 38)]));
	// hello phrase
	u8g2.setFont(u8g2_font_profont10_mr);
	u8g2.setCursor(70, 42);
	u8g2.print(splashPhrase[random(0, 36)]); // <- set the number of phrases here
}

