void setupMIDI()
{
	usbMIDI.setHandleNoteOn(onNoteOn);
	usbMIDI.setHandleNoteOff(onNoteOff);
}

void onNoteOn(byte channel, byte noteNum, byte velocity) // midi input on usbMIDI
{
	switch (channel)
	{
	case 1: // play notes
		noteFreq = note[noteNum];
		setFrequency(noteFreq);
		setAmplitude(map(float(velocity), 0.0, 127.0, 0.0, 1.0));
		break;
	case 2: // set scale and root note
		rootnote = noteNum;
		currentScale = (velocity - 1) % numScales;
		setScale(false);
		break;
	case 3: // set song/part -> note numbers under 60/c4 set the song (36 - 55 constrained), 60 and above set the part (60 - 69 constrained)
		if (noteNum < 60)
		{
			song = constrain(noteNum, 36, 55) - 36; // -36 for a value range 0 - 19 (20 songs)			
			setSong();
		}
		else
		{
			part = constrain(noteNum, 60, 69) - 60; // -60 for a value range 60 - 69 (10 parts)
			setPart();
		}
		break;
	}
}

void onNoteOff(byte channel, byte noteNum, byte velocity) // midi input on usbMIDI
{
	if (channel == 1)
		setAmplitude(0);
}

void sendCC(byte ccNum, byte value, byte channel, bool isVolCC)
{
	if (isVolCC)
		vCCVal = value;
	else
		pCCVal = value;

	if (settings.ledMode == 1)
		setCcLeds(ccNum, value, isVolCC);
}

void handleSendCC() // so we send no more than one cc message per millisecond
{
	if (noHandsPresent)
		return;

	static byte lastPCCVal = 255; // we don't want to spam the same value repeatedly
	static byte lastVCCVal = 255;
	static int16_t lastBendVal = 10000;

	if (pCCVal != 255 && pCCVal != lastPCCVal)
	{
		usbMIDI.sendControlChange(settings.pitchMidiCC, pCCVal, settings.pitchMidiChan);
		midiDIN.sendControlChange(settings.pitchMidiCC, pCCVal, settings.pitchMidiChan);
		lastPCCVal = pCCVal;
	}

	if (vCCVal != 255 && vCCVal != lastVCCVal)
	{
		usbMIDI.sendControlChange(settings.volMidiCC, vCCVal, settings.volMidiChan);
		midiDIN.sendControlChange(settings.volMidiCC, vCCVal, settings.volMidiChan);
		if (vCCVal < 3)
			killLastNote();
		else if (lastVCCVal < 3)
			sendNote(notenum);

		lastVCCVal = vCCVal;
	}

	if (bendVal != 10000 && bendVal != lastBendVal)
	{
		usbMIDI.sendPitchBend(bendVal, settings.pitchMidiChan);
		midiDIN.sendPitchBend(bendVal, settings.pitchMidiChan);
		lastBendVal = bendVal;
	}

	pCCVal = 255;
	vCCVal = 255;
	bendVal = 10000;
}

void sendBend(int16_t value)
{
	bendVal = value;
}

void sendNote(byte noteNum)
{
	// turn off the previously sent note
	killLastNote();
	if (noHandsPresent)
		return;

	// turn on the new note
	usbMIDI.sendNoteOn(noteNum, midiVelo, settings.pitchMidiChan);
	midiDIN.sendNoteOn(noteNum, midiVelo, settings.pitchMidiChan);

	lastMidiNote = noteNum;

	if (settings.ledMode == 1)
		noteOnLed();
}

void killLastNote()
{
	// turn off the last sent note
	usbMIDI.sendNoteOff(lastMidiNote, 99, settings.pitchMidiChan);
	midiDIN.sendNoteOff(lastMidiNote, 99, settings.pitchMidiChan);
}