void setup() {
	// *** AUDIO ***
	AudioMemory(20); // for some reason it's normal for Visual Studio to mark AudioMemory an error here - compiles fine though
	waveInit();
	freeverb1.roomsize(0.8);
	freeverb1.damping(0.5);
	mixer1.gain(0, 0.5); // same with gain here
	mixer1.gain(1, 0.5); // and here
	reverbInputAmp.gain(0.2);
	reverbMakeupAmp.gain(1);
	masterOutputAmp.gain(1);

	// *** TIME OF FLIGHT SENSORS ***
	// change the address of one of the sensors and init
	pinMode(0, INPUT_PULLUP);
	pinMode(xShutPin1, OUTPUT);
	pinMode(xShutPin2, OUTPUT);

	// Initialize both VL53L0X's
	// Reset all sensors by setting all of their XSHUT pins low for delay(10), then set all XSHUT high to bring out of reset
	digitalWrite(xShutPin1, LOW);
	digitalWrite(xShutPin2, LOW);
	delay(10);
	digitalWrite(xShutPin1, HIGH);
	digitalWrite(xShutPin2, HIGH);
	// Keep sensor #1 awake by keeping XSHUT pin high
	// Put all other sensors into shutdown by pulling XSHUT pins low
	digitalWrite(xShutPin1, LOW);

	// Initialize tof[0] - left
	Wire.begin();
	tof[0].init();
	tof[0].setAddress(0x30);
	tof[0].setTimeout(500);
	tof[0].startContinuous();
	digitalWrite(xShutPin1, HIGH);

	// Initialize tof[1] - right
	tof[1].init();
	tof[1].setAddress(0x29);
	tof[1].setTimeout(500);
	tof[1].startContinuous();

	// Set higher than default measurement speed
	tof[0].setMeasurementTimingBudget(20000);
	tof[1].setMeasurementTimingBudget(20000);

	// *** OLED DISPLAY ***
	u8g2.begin();
	u8g2_prepare();

	// *** LEDS ***
	pinMode(led[0], OUTPUT);
	pinMode(led[1], OUTPUT);
	pinMode(13, OUTPUT); // the built-in LED on the Teensy

	// *** BUTTONS ***
	// Make input & enable pull-up resistors on switch pins
	for (byte i = 0; i < NUMBUTTONS; i++)
		pinMode(buttons[i], INPUT_PULLUP);

	// *** POTS ***
	analogReadResolution(12);
	lockAllPots();

	// *** TIMERS ***
	updateTimer.begin(update, 1000); // update is called once every millisecond

	// *** MIDI ***
	setupMIDI(); // MIDI over USB

	// *** MIDI DIN ***
	midiDIN.begin();

	// *** MODES ***
	changeMode(mode, false);

	// *** SETTINGS ***
	if (settings.pitchTofIsRight)
	{
		pitchTofMM = &smoothTofMM[1];
		volTofMM = &smoothTofMM[0];
	}
	else
	{
		pitchTofMM = &smoothTofMM[0];
		volTofMM = &smoothTofMM[1];
	}

	settingsSavedVal = EEPROM.read(2047);
	if (settingsSavedVal != savedCheckVal) // a random value to check if settings have already been saved
		saveSettings();
	else
		loadSettings();

	// *** SCALES ***
	currentScale = 0;
	setScale(false);

	// *** ROOT ***
	setRoot(false);
}
