// *** TIME OF FLIGHT SENSORS ***
void updateTOF() // called from LOOP - read new values as fast as LOOP will allow us to
{
	bool goodReading[2] = { false, false };
	int tofTmpMM[2] = { 0, 0 };
	for (byte i = 0; i < 2; i++)
	{
		uint16_t reading = tof[i].readRangeContinuousMillimeters();
		if (reading < 500) // user is probably breaking the beam, add the latest reading to the running average
		{
			if (!noHandsPresent)
				tofReadings[i][readingsIndex] = reading;
			goodReading[i] = true;
			if (F1 && !pitchBend && pitchTofMM == &smoothTofMM[i])
			{
				pitchBend = true;
				updateOLED = true; // update the display to show the pitchbend icon
			}
		}
		else if (noHandsPresent)// user is NOT breaking the beam
		{
			if (volTofMM == &smoothTofMM[0])
			{
				tofReadings[0][readingsIndex] = 1000;
				tofReadings[1][readingsIndex] = goodMM[1];
			}
			else
			{
				tofReadings[1][readingsIndex] = 1000;
				tofReadings[0][readingsIndex] = goodMM[0];
			}
		}
		else
		{
			if (pitchTofMM == &smoothTofMM[i])
				tofReadings[i][readingsIndex] = goodMM[i]; // add the last good average to the running average
			else
				tofReadings[i][readingsIndex] = 1000;
		}
		for (byte j = 0; j < 5; j++)
			tofTmpMM[i] += tofReadings[i][j];
		tofMM[i] = tofTmpMM[i] / 5;
		if (goodReading[i])
			goodMM[i] = tofMM[i];
		if (pitchBend && bendCenterMM == 0)
			bendCenterMM = goodMM[i];
	}
	if (!goodReading[0] && !goodReading[1])
	{
		noHandsPresent = true;
		killLastNote();
	}
	else if (noHandsPresent && goodReading[0] && goodReading[1]) // both hands need to be present to get out of noHandsPresent state
	{
		noHandsPresent = false;
		//sendNote(notenum);
	}

	readingsIndex = (readingsIndex + 1 < 5) ? readingsIndex + 1 : 0;
}

void handleTOF() // called from UPDATE
{
	static volatile uint16_t lastTofMM[2];
	static volatile uint16_t targetTofMM[2];

	static uint8_t progress[2] = { smoothMS, smoothMS };
	static float increment[2];

	for (int i = 0; i < 2; i++) // apply smoothing
	{
		if (tofMM[i] != lastTofMM[i]) // only switch to the new TOF target value once moothing has been completed for the previous target value
		{
			if (progress[i] == smoothMS)
			{
				targetTofMM[i] = tofMM[i];
				progress[i] = 0;
				increment[i] = ((float)targetTofMM[i] - (float)lastTofMM[i]) / (float)smoothMS;
			}
		}
		if (progress[i] < smoothMS)
		{
			progress[i]++;
			smoothTofMM[i] = lastTofMM[i] + (int16_t)(increment[i] * progress[i]);
			if (progress[i] == smoothMS)
				lastTofMM[i] = targetTofMM[i];
		}
	}

	if (*volTofMM < settings.volMaxMM && *volTofMM > 20)
	{
		byte lastMidiVelo = midiVelo;
		midiVelo = map(*volTofMM, settings.volMaxMM, 20, 0, 127);
		if (midiVelo != lastMidiVelo)
			sendCC(settings.volMidiCC, midiVelo, settings.volMidiChan, true);
		ampGain = map((float)*volTofMM, settings.volMaxMM, 20.0, 0, maxGain);
		ampGain = CircularEaseIn(ampGain) * waveMakeUp;
		setAmplitude(ampGain);
	}
	else
		midiVelo = 0;

	if (*pitchTofMM < settings.pitchMaxMM && *pitchTofMM > 20)
	{
		rawNote = map(*pitchTofMM, 20, settings.pitchMaxMM, 0, settings.pitchNumSteps);

		if (currentScale == numScales) // free pitch / theremin mode
		{
			latestFreq = map(*pitchTofMM, 20, settings.pitchMaxMM, note[rootnote], note[rootnote + 24]);
			setFrequency(latestFreq);
			byte pitchCCVal = map(*pitchTofMM, settings.pitchMaxMM, 20, 0, 127);
			sendCC(settings.pitchMidiCC, pitchCCVal, settings.pitchMidiChan, false);
		}
		else if (pitchBend)
		{
			uint16_t bendFreq = constrain(*pitchTofMM, bendCenterMM - 20, bendCenterMM + 20);
			setFrequency(map(bendFreq, bendCenterMM - 20, bendCenterMM + 20, note[bendNote - settings.bendRange], note[bendNote + settings.bendRange]));
			sendBend(map(bendFreq, bendCenterMM - 20, bendCenterMM + 20, -8191, 8191));
		}
		// -> setAmplitude(ampGain);
	}
	else
		rawNote = 99;
}

float CircularEaseIn(float p)
{
	return 1 - sqrt(1 - (p * p));
}
