// NB: in order to compile, select MIDI from the "Tools > USB Type" menu

#include <VL53L0X.h>
#include <Audio.h>
#include <Wire.h>
#include <EEPROM.h>
#include <U8g2lib.h>
#include <SoftwareSerial.h>
#include <MIDI.h>

// *** AUDIO ***
float noteFreq;

// GUItool: begin automatically generated code
AudioSynthWaveform       waveform1;      //xy=85,321
AudioAmplifier           reverbInputAmp; //xy=256,355
AudioEffectFreeverb      freeverb1;      //xy=409,355
AudioMixer4              mixer1;         //xy=543,338
AudioAmplifier           reverbMakeupAmp; //xy=701,338
AudioAmplifier           masterOutputAmp; //xy=894.0000228881836,338.0000114440918
AudioOutputI2S           i2s1;           //xy=1060.0000305175781,339.0000104904175
AudioConnection          patchCord1(waveform1, 0, mixer1, 0);
AudioConnection          patchCord2(waveform1, reverbInputAmp);
AudioConnection          patchCord3(reverbInputAmp, freeverb1);
AudioConnection          patchCord4(freeverb1, 0, mixer1, 1);
AudioConnection          patchCord5(mixer1, reverbMakeupAmp);
AudioConnection          patchCord6(reverbMakeupAmp, masterOutputAmp);
AudioConnection          patchCord7(masterOutputAmp, 0, i2s1, 0);
AudioConnection          patchCord8(masterOutputAmp, 0, i2s1, 1);
// GUItool: end automatically generated code

int16_t current_waveform = 0;
extern const int16_t customWaveform[256];  // defined in myWaveform.ino
extern const float note[];  // defined in NOTES.ino
byte volume = 100;
float ampGain;
float maxGain = 1.0; // have to lower this for quieter waveforms that crackle otherwise 
float waveMakeUp = 0; // gain to the master amp for quiet waveforms
float reverbLevel;
const String voiceNames[9] = { "Sine", "Saw", "Square", "Triangle", "User", "Pulse", "Reverse Saw", "Sample & Hold", "Var. Triangle" };
String voiceName = "Sine";

// *** SCALES ***
const byte numScales = 21; // the number of scales we have defined in SCALES
byte *scalePointer; // a pointer variable to store the beginning of a scale array
byte scaleSize; // the size of the scale in question
int16_t currentScale; // the number of the currentscale
String scaleName;
int16_t rootnote = 40;
const String noteNames[12] = { "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };
String rootName;

// *** TIME OF FLIGHT SENSORS ***
VL53L0X tof[2]; // tof[0] is located on the left as seen from above, tof[1] on the right
const byte xShutPin1 = 21;
const byte xShutPin2 = 20;
volatile uint16_t tofMM[2];
uint16_t  tofReadings[2][5];
uint16_t goodMM[2];
byte readingsIndex = 0;
volatile uint16_t smoothTofMM[2];
volatile uint16_t *pitchTofMM; // using pointers so we can decouple from the physical tof and allow user to pick side
volatile uint16_t *volTofMM;
const uint8_t smoothMS = 20; // apply smoothing over smoothMS milliseconds
bool pitchBend = false;
uint16_t bendCenterMM;
byte bendNote;
bool noHandsPresent;

// *** OLED ***
U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE);
bool updateOLED = true;

// *** UI ***
byte mode = 0; // 0 = play, 1 = song, 3 = setup, 10 = save, 11 = save confirm
byte lastMode = 0;
const byte numModes = 3; // the number of mode we can navigate to with F1 and F2, save mode (10) is treated seprately as a F2-hold function
byte param = 0; // the parameter we're currently editing
byte lastParam[4] = { 0, 0, 0, 0 }; // keep track of the last param we were editing for a given mode - using [3] for mode 10 (save) 
byte numParams;
uint16_t alert; // number of ms to show an alert - if 0 (no alert), the current value is shown
String alertString;
uint8_t xPos;
uint8_t boxWidth;
uint8_t alertSize;
String modeName;
String paramName[3];
String *paramVal[3];
String tempVal[3];
int16_t *adjustValuePtr[3];
int16_t adjustIncrement[3];
int16_t upperLimit[3];
int16_t lowerLimit[3];
bool ignoreF1Released = false;
bool ignoreF2Released = false;
bool F1 = false;
bool F2 = false;
String blank = "";
String side[2] = { "Left", "Right" };
String ledModes[3] = { "Off",  "MIDI", "Tuning" };
int32_t saveTimer = 0;
String versionNumber = "v1.0";
bool splash = true;

// *** LEDS ***
const byte led[2] = { 11, 12 };
byte ledVal[2] = { 0, 0 };
byte ledCounter;
byte noteOnCounter;
float latestFreq; // used by the LEDs in tuning mode
int confirmBlinkCounter;  // flashes the Teensy's onboard LED 3 times to confirm firmware has been successfully uploaded

// *** BUTTONS ***
#define DEBOUNCE 10
const byte buttons[] = { 0, 1, 2, 3, 4, 5 };
#define NUMBUTTONS sizeof(buttons)
byte pressed[NUMBUTTONS], justpressed[NUMBUTTONS], justreleased[NUMBUTTONS];

// *** POTS ***
volatile uint16_t potVal[2];
volatile uint16_t tempPotVal[2];
volatile uint8_t potLocked[2] = { 0, 0 }; // a value larger than 0 means the pot is unlocked

// *** TIMERS ***
IntervalTimer displayTimer;
IntervalTimer updateTimer;

// *** DEBUG ***
float looptime;

// *** MIDI ***
byte notenum = 0;
byte lastMidiNote = 0;
byte midiVelo;
byte rawNote;
byte pCCVal = 255;
byte vCCVal = 255;
int16_t bendVal = 255;

MIDI_CREATE_INSTANCE(HardwareSerial, Serial3, midiDIN);

// *** SETTINGS ***
struct Prefs
{
	// TOF
	int16_t pitchTofIsRight = 1; // 0 = false, 1 = true - if true, the pitch is on the right, and the volume on the left
	int16_t pitchMaxMM = 400;
	int16_t volMaxMM = 200;
	int16_t pitchNumSteps = 15;
	// MIDI
	int16_t pitchMidiChan = 1;
	int16_t pitchMidiCC = 0;
	int16_t volMidiChan = 1;
	int16_t volMidiCC = 1;
	int16_t isMuted = 0; // 0 = false, 1 = true
	int16_t bendRange = 1; // in semitones
	int16_t ledMode = 2; // 0 = off, 1 = MIDI (reflects CC values and note on events), 2 = Tuning (free pitch mode only)
};

Prefs settings;
int16_t settingsSavedVal = 0;
bool settingsAreSaved = true;
const int16_t savedCheckVal = 125;

// *** SONG ***
int16_t song = 0; // 20 songs x 10 parts each x 10 bytes per part = 2000 bytes, 100 bytes per song; saved in EEPROM form byte 0 to 1999
int16_t part = 0;
int16_t lastSong = 0; // 20 songs x 10 parts each x 10 bytes per part = 2000 bytes, 100 bytes per song; saved in EEPROM form byte 0 to 1999
int16_t lastPart = 0;
int16_t songToSave = 0; // 20 songs x 10 parts each x 10 bytes per part = 2000 bytes, 100 bytes per song; saved in EEPROM form byte 0 to 1999
int16_t partToSave = 0;

struct Part
{
	byte voice;
	byte root;
	byte scale;
	byte volume;
	byte reverb;
	byte wasSaved; // use value of 1 for true
	// 6 bytes used of 10 available - in case of we can think of something else we want to save
	// we use int16_t for all assignable variables internally, even for bytes 
	// this is so we can easily assign a pointer for the buttons
	// but we should save any byte values as bytes here to save space
};
Part prt;

// *** SPLASH ***
#define TheremonSplash_width 128
#define TheremonSplash_height 64
static unsigned char TheremonSplash_bits[] = {
   0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0xff, 0x0f, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xfe, 0xff, 0xff, 0xff,
   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f,
   0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x78, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x07, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0,
   0x07, 0xff, 0xcf, 0x00, 0xf3, 0x7f, 0xfe, 0x1f, 0xff, 0x67, 0x80, 0xe3,
   0xff, 0x38, 0x60, 0xe0, 0x07, 0x60, 0xc0, 0x00, 0x33, 0x00, 0x06, 0x18,
   0x03, 0xe0, 0xc0, 0x63, 0xc0, 0x78, 0x60, 0xe0, 0x07, 0x60, 0xc0, 0x00,
   0x33, 0x00, 0x06, 0x18, 0x03, 0xe0, 0xe1, 0x63, 0xc0, 0x78, 0x60, 0xe0,
   0x07, 0x60, 0xc0, 0x00, 0x33, 0x00, 0x06, 0x18, 0x03, 0x60, 0x63, 0x63,
   0xc0, 0xd8, 0x60, 0xe0, 0x87, 0x67, 0xc0, 0x00, 0x33, 0x00, 0x06, 0x18,
   0x03, 0x60, 0x36, 0x63, 0xc0, 0x98, 0x61, 0xe0, 0xc7, 0x67, 0xc0, 0xff,
   0xf3, 0x1f, 0xfe, 0x1f, 0xff, 0x61, 0x1e, 0x63, 0xc0, 0x18, 0x63, 0xe0,
   0xc7, 0x6f, 0xc0, 0x00, 0x33, 0x00, 0xfe, 0x1f, 0x03, 0x60, 0x0c, 0x63,
   0xc0, 0x18, 0x66, 0xe0, 0xc7, 0x6f, 0xc0, 0x00, 0x33, 0x00, 0x06, 0x03,
   0x03, 0x60, 0x00, 0x63, 0xc0, 0x18, 0x6e, 0xe0, 0xc7, 0x67, 0xc0, 0x00,
   0x33, 0x00, 0x06, 0x06, 0x03, 0x60, 0x00, 0x63, 0xc0, 0x18, 0x6c, 0xe0,
   0x87, 0x63, 0xc0, 0x00, 0x33, 0x00, 0x06, 0x0c, 0x03, 0x60, 0x00, 0x63,
   0xc0, 0x18, 0x78, 0xe0, 0x07, 0x63, 0xc0, 0x00, 0xf3, 0x7f, 0x06, 0x1c,
   0xff, 0x67, 0x00, 0xe3, 0xff, 0x18, 0x70, 0xe0, 0x07, 0x03, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0,
   0x07, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xe0, 0x07, 0x03, 0xff, 0xff, 0xff, 0xff, 0x07, 0x06,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x07, 0x83, 0xff, 0xff,
   0xff, 0xff, 0x0f, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0,
   0x07, 0x83, 0x01, 0x00, 0x00, 0x00, 0x0c, 0x06, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xe0, 0x07, 0x83, 0xf1, 0x3f, 0xe0, 0x7f, 0x0c, 0x06,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x07, 0x83, 0xf9, 0xff,
   0xf0, 0xff, 0x0d, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0,
   0x07, 0x83, 0x0d, 0xec, 0x18, 0xdc, 0x0d, 0x06, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xe0, 0x07, 0xc3, 0x0d, 0xc6, 0x18, 0xcc, 0x1d, 0x06,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x07, 0xe3, 0x0d, 0xec,
   0x18, 0xdc, 0x3d, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0,
   0x07, 0xf3, 0x0d, 0xfc, 0x18, 0xf8, 0x7d, 0x06, 0xe0, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x07, 0xe0, 0x07, 0xf3, 0x0d, 0xd0, 0x18, 0xa0, 0x7d, 0x06,
   0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0x1f, 0xe0, 0x07, 0xff, 0x0d, 0xc0,
   0x18, 0x80, 0xfd, 0x07, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xe0,
   0x07, 0xfe, 0x0d, 0xc0, 0x18, 0x80, 0xfd, 0x03, 0x1c, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x70, 0xe0, 0x07, 0xf0, 0x0d, 0xc0, 0x18, 0x80, 0x7d, 0x00,
   0x0e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0xe0, 0x07, 0xe0, 0x0d, 0xc0,
   0x18, 0x80, 0x7d, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xe0,
   0x07, 0xc0, 0x0d, 0xc0, 0x18, 0x80, 0x3d, 0x00, 0x06, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xe0, 0xe0, 0x07, 0x80, 0xf9, 0xff, 0xf0, 0xff, 0x0d, 0x00,
   0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0, 0x07, 0x80, 0xf9, 0x7f,
   0xf0, 0xff, 0x0c, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0,
   0x07, 0x80, 0x01, 0x00, 0x00, 0x00, 0x0c, 0x00, 0x07, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0x07, 0x80, 0x03, 0x00, 0x00, 0x00, 0x0c, 0x00,
   0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0, 0x07, 0x00, 0xff, 0xff,
   0xff, 0xff, 0x07, 0x00, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0,
   0x07, 0x00, 0xfe, 0xff, 0xff, 0xff, 0x03, 0x00, 0x03, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0x07, 0x00, 0xcc, 0xf0, 0x7f, 0x18, 0x03, 0x80,
   0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0, 0x07, 0x00, 0xcc, 0x30,
   0x46, 0x18, 0x03, 0xc0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0,
   0x07, 0x00, 0xcc, 0x30, 0x46, 0x18, 0x03, 0xe0, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0x07, 0x00, 0xcc, 0xf0, 0x7f, 0x18, 0x03, 0xf0,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0, 0x07, 0x00, 0xcc, 0xf0,
   0x7f, 0x18, 0x03, 0x7c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0,
   0x07, 0x00, 0xcc, 0x00, 0x00, 0x18, 0x03, 0x3f, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0x07, 0x00, 0xcc, 0x00, 0x00, 0x18, 0xc3, 0x0f,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0, 0x07, 0x00, 0xcc, 0x00,
   0x00, 0x18, 0xc3, 0xff, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0,
   0x07, 0x00, 0xcc, 0xfc, 0x1f, 0x18, 0xc3, 0xff, 0x03, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0x07, 0x00, 0xcc, 0xfc, 0x1f, 0x18, 0x03, 0xf8,
   0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0, 0x07, 0x00, 0xcc, 0xcc,
   0x19, 0x18, 0x03, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xe0,
   0x07, 0x00, 0xcc, 0xcc, 0x19, 0x18, 0x03, 0x00, 0x06, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xe0, 0xe0, 0x07, 0x00, 0xcc, 0xcc, 0x19, 0x18, 0x03, 0x00,
   0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0xe0, 0x07, 0x00, 0xcc, 0xfc,
   0x1f, 0x18, 0x03, 0x00, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0xe0,
   0x07, 0x00, 0x8c, 0xff, 0xff, 0x1f, 0x03, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x3f, 0xe0, 0x07, 0x00, 0x0c, 0xff, 0xff, 0x0f, 0x03, 0x00,
   0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0x1f, 0xe0, 0x07, 0x00, 0x0c, 0x00,
   0x00, 0x00, 0x03, 0x00, 0xe0, 0xff, 0xff, 0xff, 0xff, 0xff, 0x07, 0xe0,
   0x07, 0x00, 0x0c, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xe0, 0x07, 0x00, 0xfc, 0xff, 0xff, 0xff, 0x01, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x0f, 0x00, 0xf8, 0xff,
   0xff, 0xff, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0,
   0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x7c, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3f, 0xfc, 0xff, 0xff, 0xff,
   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x1f,
   0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0xff, 0x0f };

String splashPhrase[36] = // adding or removing phrases? remember to update the number of phrases in the last code line of DISPLAY.ino
{
	"Let's go!",
	"Beauty!",
	"Tunes!",
	"Oh hey!",
	"Do it!",
	"Yeah!",
	"Ready!",
	"Sing!",
	"Shine!",
	"Stars!",
	"Shake it!",
	"Primed!",
	"It flows!",
	"Whoohoo!",
	"Exciting!",
	"Up! Up!",
	"Maestro!",
	"So good!",
	"Gold!",
	"Platinum!",
	"Fun! Fun!",
	"Superb!",
	"Marvelous",
	"Inspiring",
	"Soul!",
	"Joy! Joy!",
	"Out loud!",
	"All good!",
	"Playtime!",
	"Ola!",
	"You can!",
	"Go time!",
	"Action!",
	"Lovely!",
	"Genius!",
	"Wonderful"
};

